// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

// Workaround to access protected functions
#ifndef protected
#  define protected public
#endif

// default includes
#include <shiboken.h>
#ifndef QT_NO_VERSION_TAGGING
#  define QT_NO_VERSION_TAGGING
#endif
#include <QtCore/QDebug>
#include <pysideqenum.h>
#include <pysideqmetatype.h>
#include <pysideutils.h>
#include <feature_select.h>
QT_WARNING_DISABLE_DEPRECATED


// module include
#include "pyside6_qtmultimedia_python.h"

// main header
#include "qtaudio_wrapper.h"

// Argument includes
#include <QtMultimedia/qaudio.h>

#include <cctype>
#include <cstring>
#include <iterator>
#include <typeinfo>



// Target ---------------------------------------------------------

extern "C" {
static PyObject *Sbk_QtAudioFunc_convertVolume(PyObject *self, PyObject *args)
{
    SBK_UNUSED(self)
    PyObject *pyResult{};
    Shiboken::AutoDecRef errInfo{};
    static const char fullName[] = "PySide6.QtMultimedia.QtAudio.convertVolume";
    SBK_UNUSED(fullName)
    Shiboken::PythonContextMarker pcm;
    int overloadId = -1;
    Shiboken::Conversions::PythonToCppConversion pythonToCpp[3];
    SBK_UNUSED(pythonToCpp)
    const Py_ssize_t numArgs = PyTuple_GET_SIZE(args);
    SBK_UNUSED(numArgs)
    PyObject *pyArgs[] = {nullptr, nullptr, nullptr};

    // invalid argument lengths

    if (PyArg_UnpackTuple(args, "convertVolume", 3, 3, &(pyArgs[0]), &(pyArgs[1]), &(pyArgs[2])) == 0)
        return {};


    // Overloaded function decisor
    // 0: static QtAudio::convertVolume(float,QtAudio::VolumeScale,QtAudio::VolumeScale)->float
    if (numArgs == 3
        && (pythonToCpp[0] = Shiboken::Conversions::pythonToCppConversion(Shiboken::Conversions::PrimitiveTypeConverter<float>(), (pyArgs[0])))
        && (pythonToCpp[1] = Shiboken::Conversions::pythonToCppConversion(PepType_SETP(reinterpret_cast<SbkEnumType *>(SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_VOLUMESCALE_IDX]))->converter, (pyArgs[1])))
        && (pythonToCpp[2] = Shiboken::Conversions::pythonToCppConversion(PepType_SETP(reinterpret_cast<SbkEnumType *>(SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_VOLUMESCALE_IDX]))->converter, (pyArgs[2])))) {
        overloadId = 0; // convertVolume(float,QtAudio::VolumeScale,QtAudio::VolumeScale)
    }

    // Function signature not found.
    if (overloadId == -1)
        return Shiboken::returnWrongArguments(args, fullName, errInfo);

    // Call function/method
    {
        float cppArg0;
        pythonToCpp[0](pyArgs[0], &cppArg0);
        ::QtAudio::VolumeScale cppArg1{QtAudio::LinearVolumeScale};
        pythonToCpp[1](pyArgs[1], &cppArg1);
        ::QtAudio::VolumeScale cppArg2{QtAudio::LinearVolumeScale};
        pythonToCpp[2](pyArgs[2], &cppArg2);

        if (Shiboken::Errors::occurred() == nullptr) {
            // convertVolume(float,QtAudio::VolumeScale,QtAudio::VolumeScale)
            float cppResult = ::QtAudio::convertVolume(cppArg0, cppArg1, cppArg2);
            pyResult = Shiboken::Conversions::copyToPython(Shiboken::Conversions::PrimitiveTypeConverter<float>(), &cppResult);
        }
    }

    if (Shiboken::Errors::occurred() != nullptr || pyResult == nullptr) {
        Py_XDECREF(pyResult);
        return {};
    }
    return pyResult;
}


static const char *Sbk_QtAudio_PropertyStrings[] = {
    nullptr // Sentinel
};

static const char *Sbk_QtAudio_EnumFlagInfo[] = {
    "Error:Enum",
    "State:Enum",
    "VolumeScale:Enum",
    nullptr // Sentinel
};

static PyMethodDef Sbk_QtAudio_methods[] = {
    {"convertVolume", reinterpret_cast<PyCFunction>(Sbk_QtAudioFunc_convertVolume), METH_VARARGS|METH_STATIC, nullptr},
    {nullptr, nullptr, 0, nullptr} // Sentinel
};

} // extern "C"

static int Sbk_QtAudio_traverse(PyObject *self, visitproc visit, void *arg)
{
    return SbkObject_TypeF()->tp_traverse(self, visit, arg);
}
static int Sbk_QtAudio_clear(PyObject *self)
{
    return reinterpret_cast<PyTypeObject *>(SbkObject_TypeF())->tp_clear(self);
}
// Class Definition -----------------------------------------------
extern "C" {
static PyTypeObject *_Sbk_QtAudio_Type = nullptr;
static PyTypeObject *Sbk_QtAudio_TypeF(void)
{
    return _Sbk_QtAudio_Type;
}

static PyType_Slot Sbk_QtAudio_slots[] = {
    {Py_tp_base,        nullptr}, // inserted by introduceWrapperType
    {Py_tp_dealloc,     reinterpret_cast<void *>(Sbk_object_dealloc /* PYSIDE-832: Prevent replacement of "0" with subtype_dealloc. */)},
    {Py_tp_traverse,    reinterpret_cast<void *>(Sbk_QtAudio_traverse)},
    {Py_tp_clear,       reinterpret_cast<void *>(Sbk_QtAudio_clear)},
    {Py_tp_methods,     reinterpret_cast<void *>(Sbk_QtAudio_methods)},
    {Py_tp_new,         reinterpret_cast<void *>(SbkObject_tp_new)},
    {0, nullptr}
};
static PyType_Spec Sbk_QtAudio_spec = {
    "2:PySide6.QtMultimedia.QtAudio",
    sizeof(SbkObject),
    0,
    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
    Sbk_QtAudio_slots
};

} //extern "C"

// Type conversion functions.

// Python to C++ enum conversion.
static void QtAudio_Error_PythonToCpp_QtAudio_Error(PyObject *pyIn, void *cppOut)
{
    const auto value = static_cast<::QtAudio::Error>(Shiboken::Enum::getValue(pyIn));
    *reinterpret_cast<::QtAudio::Error *>(cppOut) = value;

}
static PythonToCppFunc is_QtAudio_Error_PythonToCpp_QtAudio_Error_Convertible(PyObject *pyIn)
{
    if (PyObject_TypeCheck(pyIn, SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_ERROR_IDX]))
        return QtAudio_Error_PythonToCpp_QtAudio_Error;
    return {};
}
static PyObject *QtAudio_Error_CppToPython_QtAudio_Error(const void *cppIn)
{
    const int castCppIn = int(*reinterpret_cast<const ::QtAudio::Error *>(cppIn));
    return Shiboken::Enum::newItem(SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_ERROR_IDX], castCppIn);

}

static void QtAudio_State_PythonToCpp_QtAudio_State(PyObject *pyIn, void *cppOut)
{
    const auto value = static_cast<::QtAudio::State>(Shiboken::Enum::getValue(pyIn));
    *reinterpret_cast<::QtAudio::State *>(cppOut) = value;

}
static PythonToCppFunc is_QtAudio_State_PythonToCpp_QtAudio_State_Convertible(PyObject *pyIn)
{
    if (PyObject_TypeCheck(pyIn, SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_STATE_IDX]))
        return QtAudio_State_PythonToCpp_QtAudio_State;
    return {};
}
static PyObject *QtAudio_State_CppToPython_QtAudio_State(const void *cppIn)
{
    const int castCppIn = int(*reinterpret_cast<const ::QtAudio::State *>(cppIn));
    return Shiboken::Enum::newItem(SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_STATE_IDX], castCppIn);

}

static void QtAudio_VolumeScale_PythonToCpp_QtAudio_VolumeScale(PyObject *pyIn, void *cppOut)
{
    const auto value = static_cast<::QtAudio::VolumeScale>(Shiboken::Enum::getValue(pyIn));
    *reinterpret_cast<::QtAudio::VolumeScale *>(cppOut) = value;

}
static PythonToCppFunc is_QtAudio_VolumeScale_PythonToCpp_QtAudio_VolumeScale_Convertible(PyObject *pyIn)
{
    if (PyObject_TypeCheck(pyIn, SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_VOLUMESCALE_IDX]))
        return QtAudio_VolumeScale_PythonToCpp_QtAudio_VolumeScale;
    return {};
}
static PyObject *QtAudio_VolumeScale_CppToPython_QtAudio_VolumeScale(const void *cppIn)
{
    const int castCppIn = int(*reinterpret_cast<const ::QtAudio::VolumeScale *>(cppIn));
    return Shiboken::Enum::newItem(SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_VOLUMESCALE_IDX], castCppIn);

}

// The signatures string for the functions.
// Multiple signatures have their index "n:" in front.
static const char *QtMultimediaQtAudio_SignatureStrings[] = {
    "PySide6.QtMultimedia.QtAudio.convertVolume(volume:float,from:PySide6.QtMultimedia.QtAudio.VolumeScale,to:PySide6.QtMultimedia.QtAudio.VolumeScale)->float",
    nullptr}; // Sentinel

void init_QtMultimediaQtAudio(PyObject *module)
{
    _Sbk_QtAudio_Type = Shiboken::ObjectType::introduceWrapperType(
        module,
        "QtAudio",
        "QtAudio",
        &Sbk_QtAudio_spec,
        nullptr,
        nullptr,
        0,
        0);
    auto *pyType = Sbk_QtAudio_TypeF(); // references _Sbk_QtAudio_Type
    InitSignatureStrings(pyType, QtMultimediaQtAudio_SignatureStrings);
    SbkObjectType_SetPropertyStrings(pyType, Sbk_QtAudio_PropertyStrings);
    SbkPySide6_QtMultimediaTypes[SBK_QtMultimediaQTAUDIO_IDX] = pyType;



    // Pass the ..._EnumFlagInfo to the class.
    SbkObjectType_SetEnumFlagInfo(pyType, Sbk_QtAudio_EnumFlagInfo);

    // Initialization of enums.
    Shiboken::AutoDecRef tpDict{};
    PyTypeObject *EType{};

    // Initialization of enum 'Error'.
    const char *Error_Initializer[] = {
        "NoError",
        "OpenError",
        "IOError",
        "UnderrunError",
        "FatalError",
        nullptr};
    // "uint8_t" used instead of "unsigned int"
    constexpr uint8_t Error_InitializerValues[] = {
        uint8_t(QtAudio::Error::NoError),       //  0x00  0
        uint8_t(QtAudio::Error::OpenError),     //  0x01  1
        uint8_t(QtAudio::Error::IOError),       //  0x02  2
        uint8_t(QtAudio::Error::UnderrunError), //  0x03  3
        uint8_t(QtAudio::Error::FatalError)     //  0x04  4
        };
    EType = Shiboken::Enum::createPythonEnum(Sbk_QtAudio_TypeF(),
        "2:PySide6.QtMultimedia.QtAudio.Error",
        Error_Initializer, Error_InitializerValues);
    SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_ERROR_IDX] = EType;
    // Register converter for enum 'QtAudio::Error'.
    {
        SbkConverter *converter = Shiboken::Conversions::createConverter(EType,
            QtAudio_Error_CppToPython_QtAudio_Error);
        Shiboken::Conversions::addPythonToCppValueConversion(converter,
            QtAudio_Error_PythonToCpp_QtAudio_Error,
            is_QtAudio_Error_PythonToCpp_QtAudio_Error_Convertible);
        Shiboken::Enum::setTypeConverter(EType, converter);
        Shiboken::Conversions::registerConverterName(converter, "QtAudio::Error");
        Shiboken::Conversions::registerConverterName(converter, "Error");
    }
    // End of 'Error' enum.

    // Initialization of enum 'State'.
    const char *State_Initializer[] = {
        "ActiveState",
        "SuspendedState",
        "StoppedState",
        "IdleState",
        nullptr};
    // "uint8_t" used instead of "unsigned int"
    constexpr uint8_t State_InitializerValues[] = {
        uint8_t(QtAudio::State::ActiveState),    //  0x00  0
        uint8_t(QtAudio::State::SuspendedState), //  0x01  1
        uint8_t(QtAudio::State::StoppedState),   //  0x02  2
        uint8_t(QtAudio::State::IdleState)       //  0x03  3
        };
    EType = Shiboken::Enum::createPythonEnum(Sbk_QtAudio_TypeF(),
        "2:PySide6.QtMultimedia.QtAudio.State",
        State_Initializer, State_InitializerValues);
    SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_STATE_IDX] = EType;
    // Register converter for enum 'QtAudio::State'.
    {
        SbkConverter *converter = Shiboken::Conversions::createConverter(EType,
            QtAudio_State_CppToPython_QtAudio_State);
        Shiboken::Conversions::addPythonToCppValueConversion(converter,
            QtAudio_State_PythonToCpp_QtAudio_State,
            is_QtAudio_State_PythonToCpp_QtAudio_State_Convertible);
        Shiboken::Enum::setTypeConverter(EType, converter);
        Shiboken::Conversions::registerConverterName(converter, "QtAudio::State");
        Shiboken::Conversions::registerConverterName(converter, "State");
    }
    // End of 'State' enum.

    // Initialization of enum 'VolumeScale'.
    const char *VolumeScale_Initializer[] = {
        "LinearVolumeScale",
        "CubicVolumeScale",
        "LogarithmicVolumeScale",
        "DecibelVolumeScale",
        nullptr};
    // "uint8_t" used instead of "unsigned int"
    constexpr uint8_t VolumeScale_InitializerValues[] = {
        uint8_t(QtAudio::VolumeScale::LinearVolumeScale),      //  0x00  0
        uint8_t(QtAudio::VolumeScale::CubicVolumeScale),       //  0x01  1
        uint8_t(QtAudio::VolumeScale::LogarithmicVolumeScale), //  0x02  2
        uint8_t(QtAudio::VolumeScale::DecibelVolumeScale)      //  0x03  3
        };
    EType = Shiboken::Enum::createPythonEnum(Sbk_QtAudio_TypeF(),
        "2:PySide6.QtMultimedia.QtAudio.VolumeScale",
        VolumeScale_Initializer, VolumeScale_InitializerValues);
    SbkPySide6_QtMultimediaTypes[SBK_QTAUDIO_VOLUMESCALE_IDX] = EType;
    // Register converter for enum 'QtAudio::VolumeScale'.
    {
        SbkConverter *converter = Shiboken::Conversions::createConverter(EType,
            QtAudio_VolumeScale_CppToPython_QtAudio_VolumeScale);
        Shiboken::Conversions::addPythonToCppValueConversion(converter,
            QtAudio_VolumeScale_PythonToCpp_QtAudio_VolumeScale,
            is_QtAudio_VolumeScale_PythonToCpp_QtAudio_VolumeScale_Convertible);
        Shiboken::Enum::setTypeConverter(EType, converter);
        Shiboken::Conversions::registerConverterName(converter, "QtAudio::VolumeScale");
        Shiboken::Conversions::registerConverterName(converter, "VolumeScale");
    }
    // End of 'VolumeScale' enum.


    // Begin code injection
    // ========================================================================
    // START of custom code block [file: ../glue/qtmultimedia.cpp (qtaudio-namespace-compatibility-alias)]
    Py_INCREF(pyType);
    PyModule_AddObject(module, "QAudio", reinterpret_cast<PyObject *>(pyType));
    // END of custom code block [file: ../glue/qtmultimedia.cpp (qtaudio-namespace-compatibility-alias)]
    // ========================================================================
    // End of code injection

    qRegisterMetaType< ::QtAudio::Error >("QtAudio::Error");
    qRegisterMetaType< ::QtAudio::State >("QtAudio::State");
    qRegisterMetaType< ::QtAudio::VolumeScale >("QtAudio::VolumeScale");
}

